
package com.reactlibrary;

import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.Callback;
import com.facebook.react.bridge.Promise;

import android.content.ComponentName;
import android.content.Intent;
import android.os.Parcel;
import android.os.ResultReceiver;
import android.os.Bundle;
import android.content.pm.PackageManager;
import android.util.Log;
import java.util.HashMap;
import java.util.Map;

public class RNPrinterZebraModule extends ReactContextBaseJavaModule {

  private final ReactApplicationContext reactContext;

  public RNPrinterZebraModule(ReactApplicationContext reactContext) {
    super(reactContext);
    this.reactContext = reactContext;
  }

  @Override
  public String getName() {
    return "RNPrinterZebra";
  }

    private String templateName = "";
    private Map<String, String> variableData = new HashMap<>();
    private int resultInt = -1;

    private boolean isAppInstalled(String packageName) {
        PackageManager pm = reactContext.getCurrentActivity().getPackageManager();
        boolean installed;

        try {
            pm.getPackageInfo(packageName, PackageManager.GET_ACTIVITIES);
            installed = true;
        } catch (PackageManager.NameNotFoundException e) {
            installed = false;
        }

        return installed;
    }


    @ReactMethod
    public void getPrinterInstance(Promise promise) {
        if(isAppInstalled("com.zebra.printconnect")){
            buildPrintDemoTemplateData();
            promise.resolve(true);
        }
        else{
            Log.d("ZT410: ", "PrintConnect Not Installed");
            promise.reject("ZT410", "PrintConnect Not Installed");
        }
    }

    @ReactMethod
    public void printLabel() {
        sendIntent();
        Log.d("ZT410: ", "Result Code: " + resultInt);
    }

    public Map<String, String> buildHeaderVarsMap() {
        Map<String, String> varMap = new HashMap<>();

        varMap.put("var_item_name", "Timaniya Set");
        varMap.put("var_barcode_no", "0019");
        varMap.put("var_tunch", "C");
        varMap.put("var_amount", "43102");
        varMap.put("var_theli_no", "2");
        varMap.put("var_stone_group", "MPM");
        varMap.put("var_total_weight", "213.21");
        varMap.put("var_gross_weight", "102.14");

        return varMap;
    }

    public void buildPrintDemoTemplateData() {
        variableData = buildHeaderVarsMap();
        templateName = "JEWvar.prn";
    }

    private void sendIntent() {
        Intent templatePrintIntent = new Intent();
        templatePrintIntent.setComponent(new ComponentName("com.zebra.printconnect", "com.zebra.printconnect.print.TemplatePrintService"));
        templatePrintIntent.putExtra("com.zebra.printconnect.PrintService.TEMPLATE_FILE_NAME", templateName);
        templatePrintIntent.putExtra("com.zebra.printconnect.PrintService.VARIABLE_DATA", (HashMap) variableData);

        templatePrintIntent.putExtra("com.zebra.printconnect.PrintService.RESULT_RECEIVER", buildIPCSafeReceiver(new ResultReceiver(null) {
            @Override
            protected void onReceiveResult(final int resultCode, final Bundle resultData) {
                if(resultCode == 0) {
                    resultInt = resultCode;
                    Log.d("ZT410: ", "Successfully Printed!");
                }
            }
        }));
        reactContext.getCurrentActivity().startService(templatePrintIntent);
    }



    public ResultReceiver buildIPCSafeReceiver(ResultReceiver actualReceiver) {
        Parcel parcel = Parcel.obtain();
        actualReceiver.writeToParcel(parcel, 0);
        parcel.setDataPosition(0);
        ResultReceiver receiverForSending = ResultReceiver.CREATOR.createFromParcel(parcel);
        parcel.recycle();
        return receiverForSending;
    }
}